import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Defines Clusters objects that contains all clusters a lattice (we pass through it) contains.
 * Each cluster then contains each cells that compose it.
 * @author Damien Deville
 *
 */
public class ClustersOld
{
	private final boolean OCCUP = true ;	// defines an occupied cell
	private final boolean EMPTY = false ;	// defines an empty cell
	// we define an ArrayList clusterList that contains Sets of cells
	// each Set of cells is a cluster and the ArrayList contains all of them
	// we prefer to use a set for the actual clusters to ensure there are not cell repetition in each cluster
	private ArrayList<Set<Cell>> clusterList ;
	
	
	/**
	 * Constructor definition.
	 * Actually populates the clusterList ArrayList from the lattice
	 * @param lattice: the lattice we want to find clusters for
	 */
	public ClustersOld(Lattice lattice)
	{
		clusterList = new ArrayList<Set<Cell>>() ;	// we create the actual ArrayList
		// we loop through the lattice and check each cell
		for(int i = 0 ; i < lattice.getSize() ; i++)
		{
			for(int j = 0 ; j < lattice.getSize() ; j++)
			{
				Cell cell = lattice.getCell(i,j) ;	// we check if each cell is occupied
				if(cell.getStatus() == OCCUP)		// in this case, we apply the following 2 methods
				{
					checkTopAndLeftNeighbors(cell, lattice) ;		// we check the top and left cells
					checkBottomAndRightNeighbors(cell, lattice) ;	// we check the bottom and right cells
				}
			}
		}
	}
	
	/**
	 * Returns the number of clusters in the lattice
	 * @return the number of clusters in the lattice
	 */
	public int getClustersNumber()
	{
		return clusterList.size();
	}
	
	/**
	 * Returns an array with the sizes of all clusters in the lattice
	 * @return an array with the sizes of all clusters in the lattice
	 */
	public int[] getClustersSize()
	{
		int[] sizes = new int[clusterList.size()] ;	// we define an array of the same size as clusterList
		int i = 0 ;
		for(Iterator<Set<Cell>> it = clusterList.iterator() ; it.hasNext() ; )
		{
			sizes[i] = it.next().size() ;	// each row of the array is the size of each Set in the ArrayList
			i++ ;
		}
		return sizes ;
	}
	
	/**
	 * Print the number of cells in each cluster and print the coordinates
	 * of each cell composing this cluster
	 */
	public void printClusterCells()
	{
		// we loop through the ArrayList of cluster Sets
		for(Iterator<Set<Cell>> it = clusterList.iterator() ; it.hasNext() ; )
		{
			Set<Cell> cluster = it.next() ;		// we get each cluster Set
			System.out.print("One cluster has " + cluster.size() + " cells which are the following: ") ;
			// we loop through each Set of cells
			for(Iterator<Cell> i = cluster.iterator() ; i.hasNext() ; )
			{
				Cell cell = i.next() ;		// we get each cell in the Set
				System.out.print("(" + cell.getI() + "," + cell.getJ() + ") ") ;
			}
			System.out.println() ;
		}
	}
	
	/**
	 * Assuming a cell belongs to a cluster, returns this cluster.
	 * If the cell does not belong to any known cluster, we return an empty cluster 
	 * @param cell: the cell we want to check
	 * @return the cluster the cell belongs to
	 */
	private Set<Cell> findCluster(Cell cell)
	{	// we define an empty cluster we can return if the cell does not belong to any known cluster 
		Set<Cell> emptyCluster = new HashSet<Cell>() ;
		// we loop through the clusterList ArrayList containing all cluster sets
		for(Iterator<Set<Cell>> it = clusterList.iterator() ; it.hasNext() ; )
		{
			Set<Cell> cluster = it.next() ;	// we get each cluster set 
			if(cluster.contains(cell)) 		// if the cluster set contains the actual cell
				return cluster ;			// we return this cluster set
		}
		return emptyCluster ;	// if the cell does not belong to any cluster, we return an empty cluster
	}
	
	/**
	 * Check if the cells at the top and left of the current cell belong to any known cluster.
	 * If they don't, we add the current cell to a new cluster set.
	 * If they belong to the same cluster, we had the current cell to this cluster set.
	 * If they belong to two distinct clusters, we merge these cluster sets and add the current cell.
	 * @param cell: the current cell we are checking
	 * @param lattice: the lattice where the cells are stored
	 */
	private void checkTopAndLeftNeighbors(Cell cell, Lattice lattice)
	{
		Cell cell_top ;		// cell at the top of the current cell
		Cell cell_left ;	// cell at the right of the current cell
		
		// we make sure that cell_top is actually not outside of the lattice
		if(cell.getI() > 0)		// if it is in the lattice
			cell_top = lattice.getCell(cell.getI()-1, cell.getJ()) ;	// we get the actual cell
		else
			cell_top = new Cell(-1, -1, EMPTY) ;	// if it is outside, we define it as an empty cell outside 
		// we make sure that cell_left is actually not outside of the lattice
		if(cell.getJ() > 0)		// if it is in the lattice
			cell_left = lattice.getCell(cell.getI(), cell.getJ()-1) ;	// we get the actual cell
		else
			cell_left = new Cell(-1, -1, EMPTY) ;	// if it is outside, we define is as an empty cell outside
		
		Set<Cell> topCluster = findCluster(cell_top) ;	// we get the cluster to which the cell at the top belongs
		Set<Cell> leftCluster = findCluster(cell_left) ;	// we get the cluster to which the cell at the left belongs
		
		// if both cell_top and cell_left are empty, we have to create a new cluster and add it to the clusterSet ArrayList
		if(cell_top.getStatus() == EMPTY && cell_left.getStatus() == EMPTY)
		{
			Set<Cell> newCluster = new HashSet<Cell>() ;	// create a new cluster Set
			newCluster.add(cell) ;			// add the current cell to this cluster
			clusterList.add(newCluster) ;	// add this cluster Set to the clusterList ArrayList
		}
		// if both cell_top and cell_left belong to the same cluster, we add the current cell to this cluster
		else if(topCluster.equals(leftCluster))
		{
			topCluster.add(cell) ;	// add the current cell to this cluster
		}
		// if cell_top and cell_left belong to different clusters (or if just one cell actually belongs to a cluster)
		// we merge the clusters, add the cell to it, add the cluster to clusterSet and remove the other cluster
		else
		{
			Set<Cell> newCluster = new HashSet<Cell>() ;	// we create a new cluster Set
			newCluster.addAll(topCluster) ;		// we add all the elements from topCluster to the new cluster Set
			newCluster.addAll(leftCluster) ;	// we add all the elements from leftCluster to the new cluster Set
			newCluster.add(cell) ;				// we add the current cell to the new cluster (it might already be in, but since this is a set, no worries)
			clusterList.add(newCluster) ;		// we had this new cluster Set to the clusterList ArrayList
			clusterList.remove(topCluster) ;	// we remove topCluster Set from the clusterList ArrayList
			clusterList.remove(leftCluster) ;	// we remove leftCluster Set from the clusterList ArrayList
		}
	}
	
	/**
	 * Check if the cells at the bottom and right of the current cell are occupied.
	 * If they are, we had them to the cluster to which the current cell belongs.
	 * If they don't, we do nothing.
	 * @param cell: the current cell we are checking
	 * @param lattice: the lattice where the cells are stored
	 */
	private void checkBottomAndRightNeighbors(Cell cell, Lattice lattice)
	{
		Set<Cell> cluster = findCluster(cell) ;	// we get the cluster set to which the current cell belongs
		if(cell.getI() < lattice.getSize()-1)	// we check that the bottom cell won't actually be outside of the lattice
		{
			Cell cell_bottom = lattice.getCell(cell.getI()+1, cell.getJ()) ;	// we get the bottom cell
			if(cell_bottom.getStatus() == OCCUP)	// if the cell is occupied
				cluster.add(cell_bottom) ;			// we add it to the current cell's cluster
		}
		if(cell.getJ() < lattice.getSize()-1)	// we check that the right cell won't actually be outside of the lattice
		{
			Cell cell_right = lattice.getCell(cell.getI(), cell.getJ()+1) ;		// we get the right cell
			if(cell_right.getStatus() == OCCUP)		// if the cell is occupied
				cluster.add(cell_right) ;			// we add it to the current cell's cluster
		}
	}
}