import java.util.Random;

/**
 * Defines lattice objects
 * @author Damien Deville
 *
 */
public class Lattice
{
	private final boolean OCCUP = true ;	// defines an occupied cell
	private final boolean EMPTY = false ;	// defines a non-occupied cell
	private int N ;							// size of the lattice
	private double prob ;					// probability of an occupied node
	private Cell[][] lattice ;				// array that represents the lattice
	private Random random ;					// random seed generator
	
	
	/**
	 * Constructor definition, creates an empty lattice of size N
	 * All the sites are defined like empty
	 * @param N : the lattice size
	 */
	public Lattice(int N)
	{
		setSize(N) ;	// we set the lattice size
		lattice = new Cell[N][N] ;	// we create the actual lattice
		for(int i = 0 ; i < N ; i++)	// we populate it given the probability
		{
			for(int j = 0 ; j < N ; j++)
			{
				lattice[i][j] = new Cell(i,j,EMPTY) ;
			}
		}
	}
	
	/**
	 * Constructor definition, creates a lattice of size N populated given the probability p.
	 * If the randomly generated number is greater than p, we set the given cell of the lattice
	 * to be occupied, otherwise we set it as empty
	 * @param N: the lattice size
	 * @param prob: the probability of each node being occupied
	 */
	public Lattice(int N, double prob)
	{
		random = new Random() ;	// we define a random object
		setSize(N) ;	// we set the lattice size
		setProb(prob) ;	// we set the probability
		lattice = new Cell[N][N] ; // we create the actual lattice
		for(int i = 0 ; i < N ; i++)	// we populate it given the probability
		{
			for(int j = 0 ; j < N ; j++)
			{
				if(random.nextDouble() <= prob)	// we set the cell to be occupied
					lattice[i][j] = new Cell(i,j,OCCUP) ;
				else							// we set the cell to be empty
					lattice[i][j] = new Cell(i,j,EMPTY) ;
			}
		}
	}
	
	/**
	 * Constructor definition, creates a lattice of size N, populated given the probability p
	 * for a particular random seed. If the randomly generated number is greater than p, we set
	 * the given cell of the lattice to be occupied, otherwise we set it as empty
	 * @param N: the lattice size
	 * @param prob: the probability of each node being occupied
	 * @param seed: a particular seed for the random generator
	 */
	public Lattice(int N, double prob, long seed)
	{
		random = new Random(seed) ;	// we define a random object given the particular seed
		setSize(N) ;	// we set the lattice size
		setProb(prob) ;	// we set the probability
		lattice = new Cell[N][N] ;	// we create the actual lattice
		for(int i = 0 ; i < N ; i++)	// we populate it given the probability
		{
			for(int j = 0 ; j < N ; j++)
			{
				if(random.nextDouble() <= prob)	// we set the cell to be occupied
					lattice[i][j] = new Cell(i,j,OCCUP) ;
				else							// we set the cell to be empty
					lattice[i][j] = new Cell(i,j,EMPTY) ;
			}
		}
	}
	
	
	
	/**
	 * Get the lattice size
	 * @return the lattice size
	 */
	public int getSize()
	{
		return N ;
	}
	
	/**
	 * Get the probability
	 * @return the probability
	 */
	public double getProb()
	{
		return prob ;
	}
	
	/**
	 * Get the cell at the i,j coordinates in the lattice
	 * @param i: the i-coordinate in the lattice
	 * @param j: the j-coordinate in the lattice
	 * @return the cell corresponding to these coordinates
	 */
	public Cell getCell(int i, int j)
	{
		return lattice[i][j] ;
	}
	
	/**
	 * Set the lattice size
	 * @param N: the lattice size
	 */
	private void setSize(int N)
	{
		this.N = N ;
	}
	
	/**
	 * Set the probability
	 * @param p: the probability
	 */
	private void setProb(double prob)
	{
		this.prob = prob ;
	}
	
	/**
	 * Set a cell to the defined position in the lattice
	 * @param cell: the cell we want to set
	 */
	public void setCell(Cell cell)
	{
		int i = cell.getI() ;
		int j = cell.getJ() ;
		lattice[i][j] = cell ;
	}
	
	/**
	 * Loop through the lattice and plot it. Plot a "*" if the cell is occupied and
	 * an empty space if it's empty
	 */
	public void plotLattice()
	{
		for(int i = 0 ; i < N ; i++)
		{
			for(int j = 0 ; j < N ; j++)
			{
				if(lattice[i][j].getStatus() == OCCUP)
					System.out.print("*") ;		// if the cell is occupied, we draw a *
				else
					System.out.print(" ") ;		// if the cell is empty, we draw an empty space
			}
			System.out.println("") ;
		}
	}
}